"use strict";
Object.defineProperties(exports, { __esModule: { value: true }, [Symbol.toStringTag]: { value: "Module" } });
const createError = require("@fastify/error");
const fp = require("fastify-plugin");
const zod = require("zod");
const _interopDefaultCompat = (e) => e && typeof e === "object" && "default" in e ? e : { default: e };
const createError__default = /* @__PURE__ */ _interopDefaultCompat(createError);
const fp__default = /* @__PURE__ */ _interopDefaultCompat(fp);
const FASTIFY_ZOD_QUERY_COERCION_PROCESSED = Symbol("FASTIFY_ZOD_QUERY_COERCION_PROCESSED");
function isZodType(schema, typeName) {
  var _a;
  return ((_a = schema == null ? void 0 : schema._def) == null ? void 0 : _a.typeName) === typeName;
}
const isValidNumber = (str) => {
  return !isNaN(Number(str)) && isFinite(Number(str));
};
const isValidDate = (str) => {
  const isoDateRegex = /^\d{4}-\d{2}-\d{2}T\d{2}:\d{2}:\d{2}(?:\.\d{0,3})?Z$/;
  return isoDateRegex.test(str);
};
class UnsupportedZodType extends Error {
  constructor(typeName) {
    super(`Unsupported schema type for query coercion: ${typeName}`);
    this.name = "UnsupportedZodType";
  }
}
function transformSchema(schema) {
  if (isZodType(schema, "ZodEffects")) {
    if (schema._def.effect.type === "preprocess") {
      return schema;
    }
    schema._def.schema = transformSchema(schema._def.schema);
    return schema;
  }
  if (isZodType(schema, "ZodLiteral")) {
    if (typeof schema._def.value === "symbol") throw new UnsupportedZodType("ZodLiteral with symbol");
    return zod.z.preprocess((val) => {
      if (typeof schema._def.value === "symbol") return val;
      if (typeof schema._def.value === "undefined") return val;
      if ("" + schema._def.value === val) return schema._def.value;
      return val;
    }, schema);
  }
  if (isZodType(schema, "ZodString")) {
    return schema;
  }
  if (isZodType(schema, "ZodNumber")) {
    return zod.z.preprocess((val) => {
      if (typeof val === "string" && isValidNumber(val)) return Number(val);
      return val;
    }, schema);
  }
  if (isZodType(schema, "ZodBigInt")) {
    return zod.z.preprocess((val) => {
      try {
        if (typeof val === "string") return BigInt(val);
        return val;
      } catch {
        return val;
      }
    }, schema);
  }
  if (isZodType(schema, "ZodBoolean")) {
    return zod.z.preprocess((val) => {
      if (val === "true") return true;
      if (val === "false") return false;
      return val;
    }, schema);
  }
  if (isZodType(schema, "ZodDate")) {
    return zod.z.preprocess((val) => {
      if (typeof val === "string") {
        if (isValidNumber(val)) return new Date(Number(val));
        if (isValidDate(val)) return new Date(val);
      }
      return val;
    }, schema);
  }
  if (isZodType(schema, "ZodEnum")) {
    return schema;
  }
  if (isZodType(schema, "ZodNull")) {
    return zod.z.preprocess((val) => {
      if (val === "null") return null;
      return val;
    }, schema);
  }
  if (isZodType(schema, "ZodNullable")) {
    schema._def.innerType = transformSchema(schema._def.innerType);
    return zod.z.preprocess((val) => {
      if (val === "null") return null;
      return val;
    }, schema);
  }
  if (isZodType(schema, "ZodOptional")) {
    schema._def.innerType = transformSchema(schema._def.innerType);
    return schema;
  }
  if (isZodType(schema, "ZodArray")) {
    schema._def.type = transformSchema(schema._def.type);
    return zod.z.preprocess((val) => {
      if (Array.isArray(val)) return val;
      return [val];
    }, schema);
  }
  if (isZodType(schema, "ZodTuple")) {
    schema._def.items = schema._def.items.map(transformSchema);
    if (schema._def.rest) {
      schema._def.rest = transformSchema(schema._def.rest);
    }
    return zod.z.preprocess((val) => {
      if (Array.isArray(val)) return val;
      return [val];
    }, schema);
  }
  if (isZodType(schema, "ZodSet")) {
    schema._def.valueType = transformSchema(schema._def.valueType);
    return zod.z.preprocess((val) => {
      if (Array.isArray(val)) return new Set(val);
      return /* @__PURE__ */ new Set([val]);
    }, schema);
  }
  if (isZodType(schema, "ZodUnion")) {
    schema._def.options = schema._def.options.map(transformSchema);
    return schema;
  }
  if (isZodType(schema, "ZodIntersection")) {
    schema._def.left = transformSchema(schema._def.left);
    schema._def.right = transformSchema(schema._def.right);
    return schema;
  }
  if (isZodType(schema, "ZodCatch")) {
    schema._def.innerType = transformSchema(schema._def.innerType);
    return schema;
  }
  if (isZodType(schema, "ZodDefault")) {
    schema._def.innerType = transformSchema(schema._def.innerType);
    return schema;
  }
  if (isZodType(schema, "ZodPipeline")) {
    schema._def.in = transformSchema(schema._def.in);
    return schema;
  }
  if (isZodType(schema, "ZodUndefined")) {
    return schema;
  }
  if (isZodType(schema, "ZodVoid")) {
    return schema;
  }
  if (isZodType(schema, "ZodAny")) {
    return schema;
  }
  if (isZodType(schema, "ZodUnknown")) {
    return schema;
  }
  if (isZodType(schema, "ZodNever")) {
    return schema;
  }
  throw new UnsupportedZodType(schema.constructor.name);
}
const FST_ZOD_QUERY_COERCION_ERROR = createError__default.default("FST_ZOD_QUERY_COERCION_ERROR", '%s at "%s"');
const plugin = async (fastify) => {
  fastify.addHook("onRoute", (route) => {
    var _a;
    if (isZodType((_a = route.schema) == null ? void 0 : _a.querystring, "ZodObject") && !route.schema.querystring[FASTIFY_ZOD_QUERY_COERCION_PROCESSED]) {
      const newShape = Object.entries(route.schema.querystring.shape).reduce((acc, [key, value]) => {
        try {
          acc[key] = transformSchema(value);
        } catch (error) {
          if (error instanceof UnsupportedZodType) {
            throw FST_ZOD_QUERY_COERCION_ERROR(error.message, key);
          }
          throw error;
        }
        return acc;
      }, {});
      route.schema.querystring._def.shape = () => newShape;
      route.schema.querystring[FASTIFY_ZOD_QUERY_COERCION_PROCESSED] = true;
    }
  });
};
const fastifyZodQueryCoercion = fp__default.default(plugin, {
  fastify: ">=4",
  name: "fastify-zod-query-coercion"
});
exports.FASTIFY_ZOD_QUERY_COERCION_PROCESSED = FASTIFY_ZOD_QUERY_COERCION_PROCESSED;
exports.default = fastifyZodQueryCoercion;
exports.fastifyZodQueryCoercion = fastifyZodQueryCoercion;
